﻿using System;
using System.Windows;
using System.Windows.Input;
using ELTE.BoardGame.Model;
using ELTE.BoardGame.Persistence;
using ELTE.BoardGame.View;
using Microsoft.Win32;

namespace ELTE.BoardGame.Controller
{
    public class BoardGameController
    {
        #region Private fields

        private BoardGameModel model; // játékmodell
        private BoardGameWindow view; // (fő) nézet
        private SaveFileDialog saveFileDialog; // dialógusok
        private OpenFileDialog openFileDialog;

        #endregion

        #region Constructors

        /// <summary>
        /// Táblajáték prezentáció példányosítása.
        /// </summary>
        /// <param name="model">A játékmodell.</param>
        public BoardGameController(BoardGameModel model)
        {
            if (model == null)
                throw new ArgumentNullException(nameof(model), "Nincs megadva játékmodell.");

            this.model = model;

            this.model.FieldChanged += new EventHandler<FieldChangedEventArgs>(Model_FieldChanged);
            this.model.GameOver += new EventHandler(Model_GameOver);
            this.model.GameWon += new EventHandler<GameWonEventArgs>(Model_GameWon);

            this.view = new BoardGameWindow();
            this.view.DataContext = CreateBoardViewModel();

            this.view.KeyDown += View_KeyDown;
            this.view.BoardClicked += View_BoardClicked;    
        }

        #endregion

        #region Public properties

        /// <summary>
        /// Nézet lekérdezése.
        /// </summary>
        public BoardGameWindow View { get { return this.view; } }

        #endregion

        #region Model event handlers

        /// <summary>
        /// Játék megnyerésének eseménykezelése.
        /// </summary>
        private void Model_GameWon(object sender, GameWonEventArgs e)
        {
            switch (e.Player)
            {
                case PlayerData.PlayerO:
                    MessageBox.Show("A kör játékos győzött!", "Játék vége!", MessageBoxButton.OK, MessageBoxImage.Asterisk);
                    break;
                case PlayerData.PlayerX:
                    MessageBox.Show("A kereszt játékos győzött!", "Játék vége!", MessageBoxButton.OK, MessageBoxImage.Asterisk);
                    break;
            }
            this.model.NewGame();
        }

        /// <summary>
        /// Játék végének eseménykezelése.
        /// </summary>
        private void Model_GameOver(object sender, EventArgs e)
        {
            MessageBox.Show("Döntetlen játék!", "Játék vége!", MessageBoxButton.OK, MessageBoxImage.Asterisk);
            this.model.NewGame();
        }

        /// <summary>
        /// Modell mezőváltozásának eseménykezelése.
        /// </summary>
        private void Model_FieldChanged(object sender, FieldChangedEventArgs e)
        {
            this.view.DataContext = this.CreateBoardViewModel();
        }

        #endregion

        #region View event handlers

        /// <summary>
        /// Táblabeli kattintás eseménykezelője.
        /// </summary>
        private void View_BoardClicked(object sender, BoardEventArgs e)
        {
            try
            {
                this.model.StepGame(e.Number / this.model.BoardWidth, e.Number % this.model.BoardWidth);
            }
            catch { }
        }

        /// <summary>
        /// Billentyűzet lenyomásának eseménykezelője.
        /// </summary>
        private void View_KeyDown(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers != ModifierKeys.Control) // ha nincs lenyomva a Control, nem csinálunk semmit
                return;

            switch (e.Key)
            {
                case Key.N: // új játék indítása (Ctrl+N)
                    this.model.NewGame();
                    break;
                case Key.L: // betöltés (Ctrl+L)
                    this.LoadGame();
                    break;
                case Key.S: // mentés (Ctrl+S)
                    this.SaveGame();
                    break;
            }
            this.view.DataContext = this.CreateBoardViewModel();
        }

        #endregion

        #region Private methods

        /// <summary>
        /// Játék betöltése.
        /// </summary>
        private void LoadGame()
        {
            if (this.openFileDialog == null)
            {
                this.openFileDialog = new OpenFileDialog();
                this.openFileDialog.Filter = "Játékmentések|*.sav";
                this.openFileDialog.Title = "Tic-Tac-Toe - Játék betöltése";
            }

            if (this.openFileDialog.ShowDialog() == true)
            {
                try
                {
                    this.model.LoadGame(this.openFileDialog.FileName);
                }
                catch (DataException)
                {
                    MessageBox.Show("Hiba keletkezett a betöltés során.", "Tic-Tac-Toe", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }
        
        /// <summary>
        /// Játék mentése.
        /// </summary>
        private void SaveGame()
        {
            if (this.saveFileDialog == null)
            {
                this.saveFileDialog = new SaveFileDialog();
                this.saveFileDialog.Filter = "Játékmentések|*.sav";
                this.saveFileDialog.Title = "Tic-Tac-Toe - Játék mentése";
            }

            if (this.saveFileDialog.ShowDialog() == true)
            {
                try
                {
                    this.model.SaveGame(this.saveFileDialog.FileName);
                }
                catch (DataException)
                {
                    MessageBox.Show("Hiba keletkezett a mentés során.", "Tic-Tac-Toe", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }
        
        /// <summary>
        /// Nézetmodell létrehozása.
        /// </summary>
        private BoardViewModel CreateBoardViewModel()
        {
            BoardItemViewModel[] items = new BoardItemViewModel[this.model.BoardHeight * this.model.BoardWidth];

            for (Int32 columnIndex = 0; columnIndex < this.model.BoardWidth; columnIndex++)
                for (Int32 rowIndex = 0; rowIndex < this.model.BoardHeight; rowIndex++)
                {
                    String player = String.Empty;

                    switch (this.model[columnIndex, rowIndex])
                    {
                        case PlayerData.NoPlayer:
                            player = " ";
                            break;
                        case PlayerData.PlayerX:
                            player = "X";
                            break;
                        case PlayerData.PlayerO:
                            player = "O";
                            break;
                    }

                    items[columnIndex * this.model.BoardWidth + rowIndex] = new BoardItemViewModel
                    {
                        Player = player,
                        Column = columnIndex,
                        Row = rowIndex,
                        Number = columnIndex * this.model.BoardWidth + rowIndex
                    };
                }

            return new BoardViewModel
            {
                BoardHeight = this.model.BoardHeight,
                BoardWidth = this.model.BoardWidth,
                BoardItems = items
            };
        }

        #endregion
    }
}
